{% extends 'flask_user/emails/base_message.html' %}

{% block message %}
<p>{{ word('Your password has been changed.') }}</p>
{% if user_manager.enable_forgot_password %}
<p>{{ word('If you did not initiate this password change,') }} <a href="{{ url_for('user.forgot_password', _external=True) }}">{{ word('click here to reset it') }}</a>.</p>
{% endif %}
{% endblock %}
