{{ start_part|safe }}
    <title>{{ page_title }}</title>
  </head>
  <body>
    <div>
      <p>{{ word("Here is some content before the interview.") }}</p>
    </div>
    <div style="width: 100%; padding: 40px">
      <div id="dablock" class="dabody dajs-embedded dahide-navbar" style="width: 100%; height: 80vh; border-style: solid; border-width: 1px; border-color: #777">
      </div>
    </div>
    <div>
      <p>{{ word("Here is some content after the interview.") }}
    </div>
    {{ scripts|safe }}
    <script src="{{ interview_url|safe }}"></script>
  </body>
</html>
