@import url(https://fonts.googleapis.com/css?family=Share+Tech+Mono);

body,
p {
  padding: 0;
  margin: 0;
}

body {
  background: #272726;
}

div#controls {
  padding: 3em;
  max-width: 1200px;
  margin: 0 auto;
}

  div#controls div {
    float: left;
  }

    div#controls div#call-controls,
    div#controls div#info {
      width: 16em;
      margin: 0 1.5em;
      text-align: center;
    }

  div#controls p.instructions {
    text-align: left;
    margin-bottom: 1em;
    font-family: Helvetica-LightOblique, Helvetica, sans-serif;
    font-style: oblique;
    font-size: 1.25em;
    color: #777776;
  }

  div#controls div#info #client-name {
    text-align: left;
    margin-bottom: 1em;
    font-family: "Helvetica Light", Helvetica, sans-serif;
    font-size: 1.25em;
    color: #777776;
  }

  div#controls button {
    width: 15em;
    height: 2.5em;
    margin-top: 1.75em;
    border-radius: 1em;
    font-family: "Helvetica Light", Helvetica, sans-serif;
    font-size: .8em;
    font-weight: lighter;
    outline: 0;
  }

    div#controls button:active {
      position: relative;
      top: 1px;
    }

  div#controls div#call-controls {
    display: none;
  }

    div#controls div#call-controls input {
      font-family: Helvetica-LightOblique, Helvetica, sans-serif;
      font-style: oblique;
      font-size: 1em;
      width: 100%;
      height: 2.5em;
      padding: .5em;
      display: block;
    }

    div#controls div#call-controls button {
      color: #fff;
      background: 0 0;
      border: 1px solid #686865;
    }

      div#controls div#call-controls button#button-hangup {
        display: none;
      }

  div#controls div#log {
    border: 1px solid #686865;
    width: 35%;
    height: 9.5em;
    margin-top: 2.75em;
    text-align: left;
    padding: 1.5em;
    float: right;
    overflow-y: scroll;
  }

    div#controls div#log p {
      color: #686865;
      font-family: 'Share Tech Mono', 'Courier New', Courier, fixed-width;
      font-size: 1.25em;
      line-height: 1.25em;
      margin-left: 1em;
      text-indent: -1.25em;
      width: 90%;
    }
