{% extends 'flask_user/emails/base_message.html' %}

{% block message %}

<p>{{ word('We have received your password reset request.') }}</p>

<p>{{ word('If you initiated this request, please click on the link below:') }}<br/>
&nbsp;&nbsp;&nbsp;&nbsp;<a href="{{ reset_password_link }}">{{ word('Reset password') }}</a>.</p>

<p>{{ word('If you did not initiate this password reset, you may safely ignore this email.') }}</p>

{% endblock %}
