{%- set footer_content = get_part('global footer') %}
{% extends 'flask_user/public_base.html' %}

{% block content %}
{% from "base_templates/form_macros.html" import render_submit_field, render_field %}
<h1>{{ title }}</h1>

<p>{{ description }}</p>

<div class="row">
  <div class="col-md-12 dacenter">
    {{ the_qrcode }}
  </div>
</div>
  
<form action="" method="POST" role="form" class="form">
  {{ form.hidden_tag() }}
  {{ render_field(form.verification_code) }}
  {{ render_submit_field(form.submit) }}
</form>

{% endblock %}
