{% extends 'flask_user/emails/base_message.html' %}

{% block message %}

<p>{{ word('An attempt was made to register an account on %s using your e-mail address.' % (app_name,)) }}</p>

<p>{{ word('If you initiated this registration, please click on the link below to sign in to your existing account:') }}<br/>
&nbsp;&nbsp;&nbsp;&nbsp;<a href="{{ sign_in_link }}">{{ word('Sign in') }}</a>.</p>

<p>{{ word('If you did not initiate this registration, you may safely ignore this email.') }}</p>

{% endblock %}
