{%- set footer_content = get_part('global footer') %}
{% extends 'flask_user/public_base.html' %}

{% block content %}
{% from "base_templates/form_macros.html" import render_submit_field %}
<h1>{{ title }}</h1>

<p>{{ description }}</p>

<form action="" method="POST" role="form" class="form">
  {{ form.hidden_tag() }}
  {{ render_submit_field(form.reconfigure, class="btn " + config['BUTTON_STYLE'] + "primary " + config['BUTTON_CLASS']) }}
{%- if allow_disable %}
  {{ render_submit_field(form.disable, class="btn " + config['BUTTON_STYLE'] + "danger " + config['BUTTON_CLASS']) }}
{%- endif %}
  {{ render_submit_field(form.cancel, class="btn " + config['BUTTON_STYLE'] + "secondary " + config['BUTTON_CLASS']) }}
</form>

{% endblock %}
